/*
 * Need to document: how to add new troff version.  Map file, and #define, for
 * those things that must be hardcoded.
 */

/*
 * trf-writer.c -- read rtf input, write troff output
 *
 * 29 Jan 91	Paul DuBois	dubois@primate.wisc.edu
 *
 * 29 Jan 91
 * - Created.
 * 31 Jan 91
 * - Made to work. :-)
 * 27 Feb 91
 * - Updated for distribution 1.05.
 * 28 Feb 91
 * - Began updating for distribution 1.06.
 * 14 Jul 93
 * - Updated to compile under THINK C 6.0.
 * 31 Aug 93
 * - Added Dave Davey's support for converting characters
 * in Symbol font to appropriate troff specials.
 * 01 Sep 93
 * - Added -h/+h options to control whether header/footer macros
 * and traps are generated, with the default being off.
 *  - Changed default troff version to GROFF.
 * 08 Sep 93
 * - Uses charset map for input character mapping and output map
 * for output mapping.
 * 11 Mar 94
 * - Updated for distribution 1.10.
 * 04 Apr 94
 * - Added defines for DWB troff and -man macros.
 *
 * troff puts paragraph markers at the beginning of paragraphs,
 * RTF puts 'em at end; the difference is a pain.
 *
 * "Small caps" is done by \s-1 ... \s+1 and capitalizing everything
 * in between.
 *
 * Good underlining is hard to do in troff; each character is
 * underlined individually, which generates an abysmal amount of
 * output.  Strikethrough output is even worse.
 *
 * ---
 *
 * Deficiencies are myriad.  They include, but are by no means
 * limited to:
 *
 * No formula support.
 *
 * Poor table support.  Tabs within cells are botched.  Tables are
 * always centered.  Tables often come out looking pretty ugly.
 *
 * space before, after and between don't appear to always work.
 *
 * troff has no decimal tabs; they are treated as (ugh) right justified
 * tabs.
 *
 * Vertical lines ("bar tabs") are unsupported.
 *
 * Poor font support (only R, I, B, not even I+B).  RTF fFont table
 * is ignored.
 *
 * Line numbering is ignored, for no good reason.
 *
 * Only normal and continuous underlining are supported.
 *
 * No outline or shadow, since troff can't do that.
 *
 * All-caps, small-caps doesn't work for non-ASCII characters (> 127)
 * or special (mapped) characters.  (Of course, why should it?)
 *
 * Default space between lines should be "auto" - but how to do that?
 *
 * Mixing of leader characters within a paragraph is not supported.
 *
 * Is nTabs = 0 handled incorrectly?
 *
 * Mechanism for handling invisible text is inconsistent.
 * It's hard to do correctly...particularly as not everything
 * is ignored when \v is in effect, and I'm not sure exactly
 * what governs whether something is ignored or not.
 *
 * Trap positions should be flushed *before* the .ev, unfortunately,
 * they might not be changed until after the .ev is already out!  Agh.
 *
 * Bottom position should be written in terms of page length, which
 * should be stored in a register.
 *
 * Need more -man, -me, -mm, -ms specific stuff.
 *
 * Need to handle \titlepg.
 */

# include	<stdio.h>
# include	<ctype.h>

# include	"rtf.h"
# include	"rtf2troff.h"


#ifndef defTroffVersion
#define defTroffVersion "groff"
#endif



static void	TokenEcho ();

static void	UnknownClass ();
static void	TextClass ();
static void	GroupClass ();
static void	ControlClass ();
static void	BeginDestination ();
static void	EndDestination ();
static void	Destination ();
static void	SpecialChar ();
static void	DocAttr ();
static void	SectAttr ();
static void	ParAttr ();
static void	CharAttr ();

static void	MapText ();

static void	SetNextTabType ();
static void	SetNextTabPos ();
static void	SetTabChar ();


static int	echo = 0;

static int	charSet = 0;			/* ansi */

char	*outMap[rtfSC_MaxChar];		/* output map */

static char	*troff = (char *) NULL;
static char	*macros = (char *) NULL;
int	tvers = NOTROFF;
int	mvers = noMacros;

int	doHeaders = 0;

int	allowUnderline = 1;
int	allowStrikeThru = 1;



State		*is;		/* current internal state */
DocState	*ids;		/* internal document state */
SectState	*iss;		/* internal section state */
ParState	*ips;		/* internal paragraph state */
CharState	*ics;		/* internal character state */

State		*ws;		/* written state */
DocState	*wds;		/* written document state */
SectState	*wss;		/* written section state */
ParState	*wps;		/* written paragraph state */
CharState	*wcs;		/* written character state */


/*
 * Whether internal state has been changed since written state
 * was last synchronized to it.  Initially set true for each file
 * processed so that internal state will be flushed when first content
 * character is written.
 */

int	docStateChanged;	/* document properties have changed */
int	sectStateChanged;	/* section properties have changed */
int	parStateChanged;	/* paragraph properties have changed */
int	charStateChanged;	/* character properties have changed */


/*
 * Indirection level for escape (\) processing.  Incremented
 * during macro/diversion collection.
 */

int	indirectionLevel = 0;


static int	haveTables = 0;	/* non-zero if any seen */
TblState	*its = (TblState *) NULL;
int		inTable = 0;	/* or not whether in table cell */


/*
 * Output stream
 */

FILE	*f;


void
SetEcho (newEcho)
int	newEcho;
{
	echo = newEcho;
}


void
SetTroffVersion (version)
char	*version;
{
	RTFFree (troff);		/* release previous version */
	if ((troff = RTFStrSave (version)) == (char *) NULL)
		RTFPanic ("SetTroffVersion: out of memory.");
}


void
SetMacroVersion (version)
char	*version;
{
	RTFFree (macros);		/* release previous version */
	if ((macros = RTFStrSave (version)) == (char *) NULL)
		RTFPanic ("SetMacroVersion: out of memory.");
}


/*
 * Once-only writer initialization
 */

void
WriterInit ()
{
	its = (TblState *) RTFAlloc ((int) sizeof (TblState));
	if (its == (TblState *) NULL)
		RTFPanic ("cannot allocate table state.");
}


/*
 * Per-file writer initialization.  The output map can change whenever
 * the macro or formatter version changes.  (To be smarter, this wouldn't
 * need to read output maps on every file if the macro/formatter are same
 * as for previous file.)
 */

int
BeginFile ()
{
char	buf[rtfBufSiz];

	f = stdout;	/* wrapper is assumed to open output on stdout */

	if (echo)
		RTFSetReadHook (TokenEcho);
	else
		RTFSetReadHook (NULL);

	/*
	 * Install writer callbacks into reader and initialize state
	 * (sets up pointers into internal state 0, and equates initial
	 * written state to internal state).
	 */

	RTFSetClassCallback (rtfText, TextClass);
	RTFSetClassCallback (rtfGroup, GroupClass);
	RTFSetClassCallback (rtfControl, ControlClass);
	RTFSetClassCallback (rtfUnknown, UnknownClass);

	charSet = 0;			/* ansi */

	docStateChanged = 1;	/* document properties have changed */
	sectStateChanged = 1;	/* section properties have changed */
	parStateChanged = 1;	/* paragraph properties have changed */
	charStateChanged = 1;	/* character properties have changed */

	indirectionLevel = 0;
	haveTables = 0;		/* set non-zero if any seen */
	inTable = 0;		/* whether or not in table cell */

	/*
	 * Determine formatter version.
	 */
	if (troff == (char *) NULL)
		SetTroffVersion (defTroffVersion);
	if (strcmp (troff, "troff") == 0)
		tvers = TROFF;
	else if (strcmp (troff, "xroff") == 0)
		tvers = XROFF;
	else if (strcmp (troff, "pstroff") == 0)
		tvers = PSTROFF;
	else if (strcmp (troff, "groff") == 0)
		tvers = GROFF;
	else if (strcmp (troff, "dwbtroff") == 0)
		tvers = DWBTROFF;
	else
	{
		tvers = NOTROFF;
		RTFMsg ("Warning: unknown formatter: %s\n", troff);
	}

	if (macros == (char *) NULL)
		mvers = noMacros;
	else if (strcmp (macros, "-man") == 0)
		mvers = manMacros;
	else if (strcmp (macros, "-me") == 0)
		mvers = meMacros;
	else if (strcmp (macros, "-mm") == 0)
		mvers = mmMacros;
	else if (strcmp (macros, "-ms") == 0)
		mvers = msMacros;
	else
	{
		mvers = noMacros;
		RTFMsg ("Warning: unknown macro package: %s\n", macros);
	}

	/*
	 * If a macro package was named, try to find an output map file for it.
	 * (If there isn't one, don't complain.)  Then read the output map for
	 * the given troff version.  The reason for reading the macro package
	 * map is that some macro packages define escape sequences for
	 * special characters.  The reason for reading it first is that macro
	 * package sequences for special characters tend to be hacked-up
	 * combinations of regular characters, and if a character can be
	 * represented by the troff version, it's likely to be better and thus
	 * should replace the macro package version.
	 */

	if (macros != (char *) NULL)
	{
		sprintf (buf, "troff%s-map", macros);
		(void) RTFReadOutputMap (buf, outMap, 1);
	}

	sprintf (buf, "%s-map", troff);
	if (RTFReadOutputMap (buf, outMap, 0) == 0)
	{
		RTFMsg ("cannot read output map %s, looking for troff-map instead\n", buf);
		if (RTFReadOutputMap ("troff-map", outMap, 0) == 0)
			RTFPanic ("cannot read output map troff-map\n");
	}

	InitState ();
	FlushInit ();

	/*
	 * If macro package needs to be told special characters
	 * might be used, do so.
	 */
	if (mvers == meMacros)
		fprintf (f, ".sc\n");
	else if (mvers == msMacros)
		fprintf (f, ".AM\n");

	/*
	 * Process the input stream.  Make sure the first intoken is
	 * a "{" so a state push will occur before anything else
	 * (need to preserve state 0 intact for section, paragraph,
	 * character default restoration).
	 */

	(void) RTFGetToken ();
	if (!RTFCheckCM (rtfGroup, rtfBeginGroup))
		RTFPanic ("malformed rtf file - does not begin with \"{\"\n");
	RTFRouteToken ();	/* send "{" through router */

	return (1);
}


void
EndFile ()
{
	Flush ();

	/* some diagnostic stuff */
	CheckFinalState ();

	if (haveTables)
		RTFMsg ("Output contains tables (run through tbl)\n");
}


/*
 * Token echo function to implement -e
 */

static void
TokenEcho ()
{
	fprintf (f, "%d\t%d\t%d\t%d\t\"%s\"\n",
			rtfClass, rtfMajor, rtfMinor, rtfParam, rtfTextBuf);
}



/* ---------------------------------------------------------------------- */

/*
 * Token class handlers
 */

/*
 * Echo any unknown tokens.  This helps indicate where the reader
 * needs to be extended.
 */

static void
UnknownClass ()
{
	RTFMsg ("Unknown symbol \"%s\" near line %ld, position %d.\n",
				rtfTextBuf, rtfLineNum, rtfLinePos);
}


/*
 * Group operator.  Push or pop internal state level.
 *
 * Before a pop, check whether the destination needs any shutdown.
 */

static void
GroupClass ()
{
	switch (rtfMajor)
	{
	case rtfBeginGroup:		/* push */
		PushIState ();
		break;
	case rtfEndGroup:		/* pop */
		/* do end-of-destination procecssing, then pop state */
		if (is->destination != rtfNoDestination)
			EndDestination ();
		PopIState ();
		break;
	}
}


static void
TextClass ()
{
	MapText (rtfMinor);
}


/*
 * Map a standard character code onto output.
 * Do uppercasing if flags are set and output sequence doesn't begin
 * with escape character.
 */

static void
MapText (stdCode)
int	stdCode;
{
char	buf[rtfBufSiz], *p;

	if (stdCode == rtfSC_nothing)
	{
		/* no entry in charset map - dump input char */
		if (rtfMajor < 128)	/* in ASCII range */
			sprintf (buf, "[[%c]]", rtfMajor);
		else
			sprintf (buf, "[[\\e'%02x]]", rtfMajor);
		PutString (buf);
		return;
	}

	p = outMap[stdCode];

	if (p != (char *) NULL
		&& *p != '\\'	/* don't uppercase escape sequences... */
		&& (ics->charStyle & (styleAllCaps | styleSmallCaps)))
	{
		(void) strcpy (buf, p);
		p = buf;
		while (*p != '\0')
		{
			if (islower (*p))
				*p = toupper (*p);
			++p;
		}
		p = buf;
	}

	/* if no output map entry was found, print standard char name */
	if (p == (char *) NULL)
	{
		sprintf (buf, "[[%s]]", RTFStdCharName (stdCode));
		p = buf;
	}
	PutString (p);
}


/*
 * The char sets, special chars and destinations do not involve
 * a state change; most other control things do.
 */

static void
ControlClass ()
{
	switch (rtfMajor)
	{
	case rtfCharSet:
		break;
	case rtfSpecialChar:
		SpecialChar ();
		break;
	case rtfDestination:
		BeginDestination ();
		break;
	case rtfDocAttr:
		DocAttr ();
		break;
	case rtfSectAttr:
		SectAttr ();
		break;
	case rtfParAttr:
		ParAttr ();
		break;
	case rtfCharAttr:
		CharAttr ();
		break;
	case rtfTblAttr:
		TblAttr ();
		break;
	}
}


/*
 * The hyphen and dash control things are treated as though they
 * were rtfText here.
 *
 * An extra level of indirection is added to the page number register.
 */

static void
SpecialChar ()
{
char	buf[rtfBufSiz];

	switch (rtfMinor)
	{
	case rtfCurHeadPage:
		PutString ("\\\\n%");	/* reference page number register */
		break;
	case rtfCurHeadDate:
		/* unimplemented */
		break;
	case rtfCurHeadDateLong:
		/* unimplemented */
		break;
	case rtfCurHeadDateAbbrev:
		/* unimplemented */
		break;
	case rtfCurHeadTime:
		/* unimplemented */
		break;
	case rtfNoBrkSpace:
		MapText (rtfSC_nobrkspace);
		break;
	case rtfNoReqHyphen:
		MapText (rtfSC_opthyphen);
		break;
	case rtfNoBrkHyphen:
		MapText (rtfSC_nobrkhyphen);
		break;
	case rtfSect:
		Sect ();
		break;
	case rtfRow:		/* end of cell, and of row/table */
		EndCell ();
		EndTbl ();
		break;
	case rtfLine:
		Par ();
		break;
	case rtfPar:
		Par ();
		break;
	case rtfCell:
		EndCell ();		/* end current cell */
		BeginCell ();		/* begin next cell */
		break;
	case rtfTab:
		PutString ("\t");
		break;
	case rtfPage:
		Par ();
		fprintf (f, ".bp\n");
		break;
	case rtfBullet:
		MapText (rtfSC_bullet);
		break;
	case rtfEmDash:
		MapText (rtfSC_emdash);
		break;
	case rtfEnDash:
		MapText (rtfSC_endash);
		break;
	case rtfLQuote:
		MapText (rtfSC_quoteleft);
		break;
	case rtfRQuote:
		MapText (rtfSC_quoteright);
		break;
	case rtfLDblQuote:
		MapText (rtfSC_quotedblleft);
		break;
	case rtfRDblQuote:
		MapText (rtfSC_quotedblright);
		break;
	}
}


/*
 * Begin/End destination don't try to do anything except dump
 * out comments delimiting the destination.  Something "real"
 * should be done, but at least the comment is better than nothing.
 *
 * The switch explicitly lists those destinations for which something
 * intelligent should be done.  (But nothing is, yet.)  Everything
 * else falls under the default case and is simply skipped anonymously.
 *
 * When a destination is skipped, the "}" is fed back into the router
 * so the group state gets popped by GroupClass().
 */

static void
BeginDestination ()
{
	Destination (1);
}


static void
EndDestination ()
{
	Destination (0);
}


static void
Destination (startDest)
int	startDest;
{
char	*dp = (char *) NULL;	/* destination name */
char	*mp = (char *) NULL;	/* macro name */
char	*rp = (char *) NULL;	/* register name */
char	*sp = (char *) NULL;	/* skipped destination name */
int	reset = 0;	/* need reset to defaults? */

	/* if beginning destination, set dest type */
	if (startDest)
		is->destination = rtfMinor;
	/* switch on destination type */
	switch (is->destination)
	{
	case rtfFootnote:
		/*
		 * Don't skip, but don't start diversion: effect is to
		 * leave footnote text in main document body.  Incorrect,
		 * but better than losing it.  Eventually, footnotes
		 * should be caught in diversions.
		 */
		dp = "footnote";
		break;
	case rtfHeader:
		dp = "header";
		mp = mHeaderAll;
		rp = rHeaderAll;
		++reset;
		break;
	case rtfHeaderLeft:
		dp = "left header";
		mp = mHeaderLeft;
		rp = rHeaderLeft;
		++reset;
		break;
	case rtfHeaderRight:
		dp = "right header";
		mp = mHeaderRight;
		rp = rHeaderRight;
		++reset;
		break;
	case rtfHeaderFirst:
		dp = "first page header";
		mp = mHeaderFirst;
		rp = rHeaderFirst;
		++reset;
		break;
	case rtfFooter:
		dp = "footer";
		mp = mFooterAll;
		rp = rFooterAll;
		++reset;
		break;
	case rtfFooterLeft:
		dp = "left footer";
		mp = mFooterLeft;
		rp = rFooterLeft;
		++reset;
		break;
	case rtfFooterRight:
		dp = "right footer";
		mp = mFooterRight;
		rp = rFooterRight;
		++reset;
		break;
	case rtfFooterFirst:
		dp = "first page footer";
		mp = mFooterFirst;
		rp = rFooterFirst;
		++reset;
		break;
	case rtfFNSep:
		sp = "footnote separator";
		break;
	case rtfFNContSep:
		sp = "continued footnote separator";
		break;
	case rtfFNContNotice:
		sp = "continued footnote notice";
		break;
	case rtfField:			/* don't ignore, but don't capture */
		dp = "field";
		break;
	case rtfFieldInst:		/* ignore */
		sp = "field instruction";
		break;
	case rtfFieldResult:		/* don't ignore, but don't capture */
		dp = "field result";
		break;
	default:
		sp = rtfTextBuf;
		break;
	}

	if (dp != (char *) NULL && startDest)
		Comment ("begin %s", dp);
	if (mp != (char *) NULL)		/* begin a capture macro */
	{
		Flush ();
		if (startDest)
		{
			FlushInitialState ();	/* make sure this is out */
			FlushSectState ();	/* flush trap positions */
			if (rp != (char *) NULL)/* set a register */
				fprintf (f, ".nr %s 1\n", rp);
			BeginDiversion (mp);
			if (reset)
			{
				/* reset paragraph, char defaults */
				/* (fake a \pard and \plain) */
				RTFSetToken (rtfControl, rtfParAttr,
					rtfParDef, rtfNoParam, "\\pard");
				RTFRouteToken ();
				RTFSetToken (rtfControl, rtfCharAttr,
					rtfPlain, rtfNoParam, "\\plain");
				RTFRouteToken ();
			}
		}
		else
		{
			EndDiversion ();
		}
	}
	if (sp != (char *) NULL)	/* skip a group */
	{
		if (startDest)
		{
			Comment ("SKIPPING %s group", sp);
			RTFSkipGroup ();
			RTFRouteToken ();	/* feed "}" back into router */
		}
		else
			Comment ("end skipped group");
	}
	if (dp != (char *) NULL && startDest == 0)
		Comment ("end %s", dp);
}


static void
DocAttr ()
{
double	inch = (double) rtfParam / (double) rtfTpi;

	switch (rtfMinor)
	{
	case rtfPaperWidth:
		ids->pageWidth = inch;
		break;
	case rtfPaperHeight:
		ids->pageHeight = inch;
		break;
	case rtfLeftMargin:
		ids->leftMargin = inch;
		break;
	case rtfRightMargin:
		ids->rightMargin = inch;
		break;
	case rtfTopMargin:
		ids->topMargin = inch;
		/* pre-emptive strike */
		if (iss->headerPos >= ids->topMargin)
			iss->headerPos = ids->topMargin / 2;
		break;
	case rtfBottomMargin:
		ids->bottomMargin = inch;
		if (iss->footerPos >= ids->bottomMargin)
			iss->footerPos = ids->bottomMargin / 2;
		break;
	case rtfDefTab:
		ids->tabWidth = inch;
		break;
	case rtfLandscape:
		ids->landscape = 1;
		break;
	}
	++docStateChanged;
}


static void
SectAttr ()
{
double	inch = (double) rtfParam / (double) rtfTpi;

	switch (rtfMinor)
	{
	case rtfSectDef:
		RestoreSectDefaults ();
		break;
	case rtfNoBreak:
	case rtfColBreak:
	case rtfPageBreak:
	case rtfEvenBreak:
	case rtfOddBreak:
		iss->breakType = rtfMinor;
		break;
	case rtfPageStarts:
		iss->pageStart = rtfParam;
		break;
	case rtfPageCont:
		iss->pageRestart = 0;
		break;
	case rtfPageRestart:
		iss->pageRestart = 1;
		break;
	case rtfPageDecimal:
		Flush ();
		fprintf (f, ".af %% 1\n");
		break;
	case rtfPageURoman:
		Flush ();
		fprintf (f, ".af %% I\n");
		break;
	case rtfPageLRoman:
		Flush ();
		fprintf (f, ".af %% i\n");
		break;
	case rtfPageULetter:
		Flush ();
		fprintf (f, ".af %% A\n");
		break;
	case rtfPageLLetter:
		Flush ();
		fprintf (f, ".af %% a\n");
		break;
	case rtfPageNumRight:
		break;
	case rtfPageNumTop:
		break;
	case rtfHeaderY:
		iss->headerPos = inch;
		if (iss->headerPos >= ids->topMargin)
			iss->headerPos = ids->topMargin / 2;
		break;
	case rtfFooterY:
		iss->footerPos = inch;
		if (iss->footerPos >= ids->bottomMargin)
			iss->footerPos = ids->bottomMargin / 2;
		break;
	case rtfLineModulus:
		break;
	case rtfLineDist:
		break;
	case rtfLineStarts:
		break;
	case rtfLineRestart:
		break;
	case rtfLineRestartPg:
		break;
	case rtfLineCont:
		break;
	case rtfTopVAlign:
		break;
	case rtfBottomVAlign:
		break;
	case rtfCenterVAlign:
		break;
	case rtfJustVAlign:
		break;
	case rtfColumns:
		break;
	case rtfColumnSpace:
		break;
	case rtfColumnLine:
		break;
	case rtfENoteHere:
		break;
	case rtfTitleSpecial:
		iss->titleSpecial = 1;
		break;
	}
	++sectStateChanged;
}


/*
 * Paragraph defaults are restored by using the state 0 values,
 * they applying the "Normal" style (style 0).
 *
 * For the rtfStyleNum, the tab flag is reset before expanding the
 * style so any inherited tabs will be overridden by tabs in the
 * style, and reset after expansion so any tabs in the paragraph
 * itself will override inherited or style tabs.
 *
 * The "unimplemented" cases below are those which are currently
 * ignored, but for which something might be done someday, i.e.,
 * they're reminders.
 */

static void
ParAttr ()
{
double	inch = (double) rtfParam / (double) rtfTpi;

	switch (rtfMinor)
	{
	case rtfParDef:
		RestoreParDefaults ();
		break;
	case rtfStyleNum:
		ips->tabFlag = 0;
		RTFExpandStyle (rtfParam);
		ips->tabFlag = 0;
		break;
	case rtfQuadLeft:
	case rtfQuadRight:
	case rtfQuadJust:
	case rtfQuadCenter:
		ips->justification = rtfMinor;
		break;
	case rtfFirstIndent:
		ips->firstIndent = inch;
		break;
	case rtfLeftIndent:
		ips->leftIndent = inch;
		break;
	case rtfRightIndent:
		ips->rightIndent = inch;
		break;
	case rtfSpaceBefore:
		ips->spaceBefore = inch;
		break;
	case rtfSpaceAfter:
		ips->spaceAfter = inch;
		break;
	case rtfSpaceBetween:
		ips->spaceBetween = inch;
		break;
	case rtfInTable:
		haveTables = 1;
		/*
		 * If first cell of row, set temp indent to left edge
		 * of table.  (Actually, this is done incorrectly; tables
		 * are always centered.)
		 * Subsequent cells are begun when \cell is seen.
		 */
		if (its->tableHeader == 0)	/* first cell; need */
		{				/* table prolog */
			BeginTbl ();
			BeginCell ();
		}
		break;
	case rtfNoLineNum:
		/* unimplemented */
		break;
	case rtfTabPos:
		SetNextTabPos (inch);
		break;
	case rtfTabRight:
	case rtfTabCenter:
	case rtfTabDecimal:
		SetNextTabType (rtfMinor);
		break;
	case rtfTabBar:
		/* unimplemented */
		break;
	case rtfBorderTop:
		ips->borderFlags |= borderTop;
		break;
	case rtfBorderBottom:
		ips->borderFlags |= borderBottom;
		break;
	case rtfBorderLeft:
		/* unimplemented */
		break;
	case rtfBorderRight:
		/* unimplemented */
		break;
	case rtfBorderBar:
		/* unimplemented */
		break;
	case rtfBorderBox:
		/* unimplemented */
		break;
	case rtfBorderBetween:
		/* unimplemented */
		break;
	case rtfBorderSingle:
	case rtfBorderThick:
	case rtfBorderShadow:
	case rtfBorderDouble:
	case rtfBorderDot:
	case rtfBorderHair:
		ips->borderType = rtfMinor;
		break;
	case rtfBorderSpace:
		/* unimplemented */
		break;
	case rtfLeaderDot:
	case rtfLeaderHyphen:
	case rtfLeaderUnder:
	case rtfLeaderThick:
		SetTabChar (rtfMinor);
		break;
	}
	++parStateChanged;
}


/*
 * Several of the attributes can be turned off with param value
 * of zero (e.g., \b vs. \b0).
 *
 * \plain is like \pard but for characters, i.e, it restores all
 * character defaults.
 */

static void
CharAttr ()
{
int	turnOn = (rtfParam != 0);

	switch (rtfMinor)
	{
	case rtfPlain:
		RestoreCharDefaults ();
		break;
	case rtfBold:
		if (turnOn)
			ics->charStyle |= styleBold;
		else
			ics->charStyle &= ~styleBold;
		break;
	case rtfItalic:
		if (turnOn)
			ics->charStyle |= styleItalic;
		else
			ics->charStyle &= ~styleItalic;
		break;
	case rtfStrikeThru:
		if (allowStrikeThru)
		{
			if (turnOn)
				ics->charStyle |= styleStrikeThru;
			else
				ics->charStyle &= ~styleStrikeThru;
		}
		break;
	case rtfOutline:
		if (turnOn)
			ics->charStyle |= styleOutline;
		else
			ics->charStyle &= ~styleOutline;
		break;
	case rtfShadow:
		if (turnOn)
			ics->charStyle |= styleShadow;
		else
			ics->charStyle &= ~styleShadow;
		break;
	case rtfSmallCaps:
		if (turnOn)
			ics->charStyle |= styleSmallCaps;
		else
			ics->charStyle &= ~styleSmallCaps;
		break;
	case rtfAllCaps:
		if (turnOn)
			ics->charStyle |= styleAllCaps;
		else
			ics->charStyle &= ~styleAllCaps;
		break;
	case rtfInvisible:
		if (turnOn)
			ics->charStyle |= styleInvisible;
		else
			ics->charStyle &= ~styleInvisible;
		break;
	case rtfFontNum:
		/* unimplemented */
		break;
	case rtfFontSize:
		/* sizes are in half-points, convert to whole points */
		ics->fontSize = (int) (rtfParam / 2);
		if (ics->fontSize <= 0)
			++ics->fontSize;	/* don't play with fire */
		break;
	case rtfExpand:
		/* unimplemented */
		break;
	case rtfUnderline:
		if (allowUnderline)
		{
			if (turnOn)
				ics->charStyle |= styleUnderline;
			else
				ics->charStyle &= ~styleUnderline;
		}
		break;
	case rtfWordUnderline:
		if (allowUnderline)
		{
			if (turnOn)
				ics->charStyle |= styleWUnderline;
			else
				ics->charStyle &= ~styleWUnderline;
		}
		break;
	case rtfDotUnderline:
		/* unimplemented */
		break;
	case rtfDbUnderline:
		/* unimplemented */
		break;
	case rtfNoUnderline:
		ics->charStyle &= ~styleUnderline;
		ics->charStyle &= ~styleWUnderline;
		break;
	case rtfSuperScript:
		/* superscripts are in half-points, convert to points */
		ics->superScript = rtfParam / 2;
		break;
	case rtfSubScript:
		/* subscripts are in half-points, convert to points */
		ics->subScript = rtfParam / 2;
		break;
	case rtfRevised:
		/* unimplemented */
		break;
	case rtfForeColor:
		/* unimplemented */
		break;
	case rtfBackColor:
		/* unimplemented */
		break;
	}
	++charStateChanged;
}


/* ---------------------------------------------------------------------- */

/*
 * Tab handling routines
 */

void
InitTabSet ()
{
int	i;

	ips->nTabs = 0;
	/*
	 * Set all tabs to be left-justified, the default that will then be
	 * used if  no other tab type is specified.  (The position is
	 * specified *after* the type; if no type is specified, use the
	 * default.)
	 */
	for (i = 0; i < maxTab; i++)
		ips->tabType[i] = rtfTabLeft;
}


static void
SetNextTabPos (pos)
double	pos;
{
	if (ips->tabFlag != 0 && ips->nTabs >= maxTab)
		RTFMsg ("maximum tabstop count (%d) exceeded\n",
							maxTab);
	else
	{
		/* if no tab info has been set for this state, reinit them */
		if (ips->tabFlag == 0)
		{
			InitTabSet ();
			ips->tabFlag = 1;
		}
		ips->tab[ips->nTabs++] = pos;
	}
}


/*
 * Tab types are specified *before* the position to which they apply
 * is given, so set the next available slot in anticipation of the
 * position's being specified next.
 */

static void
SetNextTabType (type)
int	type;
{
	if (ips->tabFlag != 0 && ips->nTabs >= maxTab)
		RTFMsg ("maximum tabstop count (%d) exceeded\n",
							maxTab);
	else
	{
		/* if no tab info has been set for this state, reinit them */
		if (ips->tabFlag == 0)
		{
			InitTabSet ();
			ips->tabFlag = 1;
		}
		ips->tabType[ips->nTabs] = type;
	}
}


static void
SetTabChar (leader)
int	leader;
{
	if (ips->tabFlag != 0 && ips->nTabs >= maxTab)
		RTFMsg ("maximum tabstop count (%d) exceeded\n",
							maxTab);
	else
	{
		/* if no tab info has been set for this state, reinit them */
		if (ips->tabFlag == 0)
		{
			InitTabSet ();
			ips->tabFlag = 1;
		}
		ips->tabChar = rtfMinor;
	}
}
