/*
 * skel-writer.c -- RTF skeleton translator writer code.
 *
 * rtfskel provides placeholders for (almost) all the tokens listed in
 * rtf.h.  You can use this as a skeleton or framework on which to
 * build a new RTF translator.  Plug in your code as you like, to
 * handle particular tokens, or rip out code for tokens you want to
 * ignore.
 * 
 * The built-in destination readers are left enabled, as is the
 * default of style expansion ON.
 * 
 * 07 Feb 91	Paul DuBois	dubois@primate.wisc.edu
 * 
 * 07 Feb 91 Created.
 * 26 Feb 91
 * - Updated for distribution 1.05.
 * 14 Jul 93
 * - Updated to compile under THINK C 6.0.  Added cases for
 * all the new symbols recognized by distribution release 1.07.
 * 22 Sep 93
 * - Added stuff for 1.09 character mapping scheme.
 * 11 Mar 94
 * - Updated for 1.10 distribution.
 */

# include	<stdio.h>

# include	"rtf.h"
# include	"rtfskel.h"


static void	UnknownClass ();
static void	TextClass ();
static void	ControlClass ();
static void	Destination ();
static void	SpecialChar ();
static void	DocAttr ();
static void	SectAttr ();
static void	TblAttr ();
static void	ParAttr ();
static void	CharAttr ();
static void	PictAttr ();
static void	BookmarkAttr ();
static void	NeXTGrAttr ();
static void	FieldAttr ();
static void	TOCAttr ();
static void	PosAttr ();
static void	ObjAttr ();
static void	FNoteAttr ();
static void	KeyCodeAttr ();
static void	ACharAttr ();
static void	FontAttr ();
static void	FileAttr ();
static void	FileSource ();
static void	DrawAttr ();
static void	IndexAttr ();

static void	SelectLanguage();


/*
 * Initialize the writer.
 */

void
WriterInit ()
{
	/* perform once-only initializations here */
}


/*
 * Terminate the writer.
 */

void
WriterEnd ()
{
	/* perform once-only cleanup here */
}


int
BeginFile ()
{
	/* install class callbacks */

	RTFSetClassCallback (rtfUnknown, UnknownClass);
	RTFSetClassCallback (rtfText, TextClass);
	RTFSetClassCallback (rtfControl, ControlClass);

	return (1);
}


void
EndFile ()
{
	/* input file exhausted - write any final output here */
}


/*
 * Token class callbacks
 */


static void
UnknownClass ()
{
	/* use this to print warnings about unknown tokens */
}

static void
TextClass ()
{
	/*
	 * rtfMajor contains character value.  However, instead of
	 * writing it directly, map it through the output map and
	 * write the result.
	 */
}


/*
 * Process control symbol.
 */

static void
ControlClass ()
{
	switch (rtfMajor)
	{
	case rtfVersion:
		break;
	case rtfDefFont:
		/*
		 * rtfParam contains the default font number.  Beware
		 * that there is no guarantee that this font will actually
		 * be listed in the font table!
		 */
		break;
	case rtfCharSet:
		/* handled by reader */
		break;
	case rtfDestination:
		Destination ();
		break;
	case rtfFontFamily:
		/* only occurs within font table - can ignore */
		break;
	case rtfColorName:
		/* only occurs within color table - can ignore */
		break;
	case rtfSpecialChar:
		SpecialChar ();
		break;
	case rtfStyleAttr:
		/* only occurs within stylesheet - can ignore */
		break;
	case rtfDocAttr:
		DocAttr ();
		break;
	case rtfSectAttr:
		SectAttr ();
		break;
	case rtfTblAttr:
		TblAttr ();
		break;
	case rtfParAttr:
		ParAttr ();
		break;
	case rtfCharAttr:
		CharAttr ();
		break;
	case rtfPictAttr:
		PictAttr ();
		break;
	case rtfBookmarkAttr:
		BookmarkAttr ();
		break;
	case rtfNeXTGrAttr:
		NeXTGrAttr ();
		break;
	case rtfFieldAttr:
		FieldAttr ();
		break;
	case rtfTOCAttr:
		TOCAttr ();
		break;
	case rtfPosAttr:
		PosAttr ();
		break;
	case rtfObjAttr:
		ObjAttr ();
		break;
	case rtfFNoteAttr:
		FNoteAttr ();
		break;
	case rtfKeyCodeAttr:
		KeyCodeAttr ();
		break;
	case rtfACharAttr:
		ACharAttr ();
		break;
	case rtfFontAttr:
		FontAttr ();
		break;
	case rtfFileAttr:
		FileAttr ();
		break;
	case rtfFileSource:
		FileSource ();
		break;
	case rtfDrawAttr:
		DrawAttr ();
		break;
	case rtfIndexAttr:
		IndexAttr ();
		break;
	}
}


/*
 * Control class major number handlers.  Each one switches on
 * the minor numbers that occur within the major number.
 */


static void
Destination ()
{
	switch (rtfMinor)
	{
	case rtfFontTbl:
		/* will never occur because of default destination reader */
		break;
	case rtfFontAltName:
		break;
	case rtfEmbeddedFont:
		break;
	case rtfFontFile:
		break;
	case rtfFileTbl:
		break;
	case rtfFileInfo:
		break;
	case rtfColorTbl:
		/* will never occur because of default destination reader */
		break;
	case rtfStyleSheet:
		/* will never occur because of default destination reader */
		break;
	case rtfKeyCode:
		/*
		 * Currently will never occur because the stylesheet
		 * reader eats the keycode group.
		 */
		break;
	case rtfRevisionTbl:
		break;
	case rtfInfo:
		break;
	case rtfITitle:
		break;
	case rtfISubject:
		break;
	case rtfIAuthor:
		break;
	case rtfIOperator:
		break;
	case rtfIKeywords:
		break;
	case rtfIComment:
		break;
	case rtfIVersion:
		break;
	case rtfIDoccomm:
		break;
	case rtfIVerscomm:
		break;
	case rtfNextFile:
		break;
	case rtfTemplate:
		break;
	case rtfFNSep:
		break;
	case rtfFNContSep:
		break;
	case rtfFNContNotice:
		break;
	case rtfENSep:
		break;
	case rtfENContSep:
		break;
	case rtfENContNotice:
		break;
	case rtfPageNumLevel:
		break;
	case rtfParNumLevelStyle:
		break;
	case rtfHeader:
		break;
	case rtfFooter:
		break;
	case rtfHeaderLeft:
		break;
	case rtfHeaderRight:
		break;
	case rtfHeaderFirst:
		break;
	case rtfFooterLeft:
		break;
	case rtfFooterRight:
		break;
	case rtfFooterFirst:
		break;
	case rtfParNumText:
		break;
	case rtfParNumbering:
		break;
	case rtfParNumTextAfter:
		break;
	case rtfParNumTextBefore:
		break;
	case rtfBookmarkStart:
		break;
	case rtfBookmarkEnd:
		break;
	case rtfPict:
		break;
	case rtfObject:
		break;
	case rtfObjClass:
		break;
	case rtfObjName:
		break;
	case rtfObjTime:
		break;
	case rtfObjData:
		break;
	case rtfObjAlias:
		break;
	case rtfObjSection:
		break;
	case rtfObjResult:
		break;
	case rtfObjItem:
		break;
	case rtfObjTopic:
		break;
	case rtfDrawObject:
		break;
	case rtfFootnote:
		break;
	case rtfAnnotRefStart:
		break;
	case rtfAnnotRefEnd:
		break;
	case rtfAnnotID:
		break;
	case rtfAnnotAuthor:
		break;
	case rtfAnnotation:
		break;
	case rtfAnnotRef:
		break;
	case rtfAnnotTime:
		break;
	case rtfAnnotIcon:
		break;
	case rtfField:
		break;
	case rtfFieldInst:
		break;
	case rtfFieldResult:
		break;
	case rtfDataField:
		break;
	case rtfIndex:
		break;
	case rtfIndexText:
		break;
	case rtfIndexRange:
		break;
	case rtfTOC:
		break;
	case rtfNeXTGraphic:
		break;
	}
}


static void
SpecialChar ()
{
	switch (rtfMinor)
	{
	case rtfIIntVersion:
		break;
	case rtfICreateTime:
		break;
	case rtfIRevisionTime:
		break;
	case rtfIPrintTime:
		break;
	case rtfIBackupTime:
		break;
	case rtfIEditTime:
		break;
	case rtfIYear:
		break;
	case rtfIMonth:
		break;
	case rtfIDay:
		break;
	case rtfIHour:
		break;
	case rtfIMinute:
		break;
	case rtfISecond:
		break;
	case rtfINPages:
		break;
	case rtfINWords:
		break;
	case rtfINChars:
		break;
	case rtfIIntID:
		break;
	case rtfCurHeadDate:
		break;
	case rtfCurHeadDateLong:
		break;
	case rtfCurHeadDateAbbrev:
		break;
	case rtfCurHeadTime:
		break;
	case rtfCurHeadPage:
		break;
	case rtfSectNum:
		break;
	case rtfCurFNote:
		break;
	case rtfCurAnnotRef:
		break;
	case rtfFNoteSep:
		break;
	case rtfFNoteCont:
		break;
	case rtfCell:
		break;
	case rtfRow:
		break;
	case rtfPar:
		break;
	case rtfSect:
		break;
	case rtfPage:
		break;
	case rtfColumn:
		break;
	case rtfLine:
		break;
	case rtfSoftPage:
		break;
	case rtfSoftColumn:
		break;
	case rtfSoftLine:
		break;
	case rtfSoftLineHt:
		break;
	case rtfTab:
		break;
	case rtfEmDash:
		break;
	case rtfEnDash:
		break;
	case rtfEmSpace:
		break;
	case rtfEnSpace:
		break;
	case rtfBullet:
		break;
	case rtfLQuote:
		break;
	case rtfRQuote:
		break;
	case rtfLDblQuote:
		break;
	case rtfRDblQuote:
		break;
	case rtfFormula:
		break;
	case rtfNoBrkSpace:
		break;
	case rtfNoReqHyphen:
		break;
	case rtfNoBrkHyphen:
		break;
	case rtfOptDest:
		break;
	case rtfLTRMark:
		break;
	case rtfRTLMark:
		break;
	case rtfNoWidthJoiner:
		break;
	case rtfNoWidthNonJoiner:
		break;
	case rtfCurHeadPict:
		break;
	}
}


static void
DocAttr ()
{
	switch (rtfMinor)
	{
	case rtfDefTab:
		break;
	case rtfHyphHotZone:
		break;
	case rtfHyphConsecLines:
		break;
	case rtfHyphCaps:
		break;
	case rtfHyphAuto:
		break;
	case rtfLineStart:
		break;
	case rtfFracWidth:
		break;
	case rtfMakeBackup:
		break;
	case rtfRTFDefault:
		break;
	case rtfPSOverlay:
		break;
	case rtfDocTemplate:
		break;
	case rtfDefLanguage:
		break;
	case rtfFENoteType:
		break;
	case rtfFNoteEndSect:
		break;
	case rtfFNoteEndDoc:
		break;
	case rtfFNoteText:
		break;
	case rtfFNoteBottom:
		break;
	case rtfENoteEndSect:
		break;
	case rtfENoteEndDoc:
		break;
	case rtfENoteText:
		break;
	case rtfENoteBottom:
		break;
	case rtfFNoteStart:
		break;
	case rtfENoteStart:
		break;
	case rtfFNoteRestartPage:
		break;
	case rtfFNoteRestart:
		break;
	case rtfFNoteRestartCont:
		break;
	case rtfENoteRestart:
		break;
	case rtfENoteRestartCont:
		break;
	case rtfFNoteNumArabic:
		break;
	case rtfFNoteNumLLetter:
		break;
	case rtfFNoteNumULetter:
		break;
	case rtfFNoteNumLRoman:
		break;
	case rtfFNoteNumURoman:
		break;
	case rtfFNoteNumChicago:
		break;
	case rtfENoteNumArabic:
		break;
	case rtfENoteNumLLetter:
		break;
	case rtfENoteNumULetter:
		break;
	case rtfENoteNumLRoman:
		break;
	case rtfENoteNumURoman:
		break;
	case rtfENoteNumChicago:
		break;
	case rtfPaperWidth:
		break;
	case rtfPaperHeight:
		break;
	case rtfPaperSize:
		break;
	case rtfLeftMargin:
		break;
	case rtfRightMargin:
		break;
	case rtfTopMargin:
		break;
	case rtfBottomMargin:
		break;
	case rtfFacingPage:
		break;
	case rtfGutterWid:
		break;
	case rtfMirrorMargin:
		break;
	case rtfLandscape:
		break;
	case rtfPageStart:
		break;
	case rtfWidowCtrl:
		break;
	case rtfLinkStyles:
		break;
	case rtfNoAutoTabIndent:
		break;
	case rtfWrapSpaces:
		break;
	case rtfPrintColorsBlack:
		break;
	case rtfNoExtraSpaceRL:
		break;
	case rtfNoColumnBalance:
		break;
	case rtfCvtMailMergeQuote:
		break;
	case rtfSuppressTopSpace:
		break;
	case rtfSuppressPreParSpace:
		break;
	case rtfCombineTblBorders:
		break;
	case rtfTranspMetafiles:
		break;
	case rtfSwapBorders:
		break;
	case rtfShowHardBreaks:
		break;
	case rtfFormProtected:
		break;
	case rtfAllProtected:
		break;
	case rtfFormShading:
		break;
	case rtfFormDisplay:
		break;
	case rtfPrintData:
		break;
	case rtfRevProtected:
		break;
	case rtfRevisions:
		break;
	case rtfRevDisplay:
		break;
	case rtfRevBar:
		break;
	case rtfAnnotProtected:
		break;
	case rtfRTLDoc:
		break;
	case rtfLTRDoc:
		break;
	}
}


static void
SectAttr ()
{
	switch (rtfMinor)
	{
	case rtfSectDef:
		break;
	case rtfENoteHere:
		break;
	case rtfPrtBinFirst:
		break;
	case rtfPrtBin:
		break;
	case rtfSectStyleNum:
		break;
	case rtfNoBreak:
		break;
	case rtfColBreak:
		break;
	case rtfPageBreak:
		break;
	case rtfEvenBreak:
		break;
	case rtfOddBreak:
		break;
	case rtfColumns:
		break;
	case rtfColumnSpace:
		break;
	case rtfColumnNumber:
		break;
	case rtfColumnSpRight:
		break;
	case rtfColumnWidth:
		break;
	case rtfColumnLine:
		break;
	case rtfLineModulus:
		break;
	case rtfLineDist:
		break;
	case rtfLineStarts:
		break;
	case rtfLineRestart:
		break;
	case rtfLineRestartPg:
		break;
	case rtfLineCont:
		break;
	case rtfSectPageWid:
		break;
	case rtfSectPageHt:
		break;
	case rtfSectMarginLeft:
		break;
	case rtfSectMarginRight:
		break;
	case rtfSectMarginTop:
		break;
	case rtfSectMarginBottom:
		break;
	case rtfSectMarginGutter:
		break;
	case rtfSectLandscape:
		break;
	case rtfTitleSpecial:
		break;
	case rtfHeaderY:
		break;
	case rtfFooterY:
		break;
	case rtfPageStarts:
		break;
	case rtfPageCont:
		break;
	case rtfPageRestart:
		break;
	case rtfPageNumRight:
		break;
	case rtfPageNumTop:
		break;
	case rtfPageDecimal:
		break;
	case rtfPageURoman:
		break;
	case rtfPageLRoman:
		break;
	case rtfPageULetter:
		break;
	case rtfPageLLetter:
		break;
	case rtfPageNumHyphSep:
		break;
	case rtfPageNumSpaceSep:
		break;
	case rtfPageNumColonSep:
		break;
	case rtfPageNumEmdashSep:
		break;
	case rtfPageNumEndashSep:
		break;
	case rtfTopVAlign:
		break;
	case rtfBottomVAlign:
		break;
	case rtfCenterVAlign:
		break;
	case rtfJustVAlign:
		break;
	case rtfRTLSect:
		break;
	case rtfLTRSect:
		break;
	}
}


static void
TblAttr ()
{
	switch (rtfMinor)
	{
	case rtfRowDef:
		break;
	case rtfRowGapH:
		break;
	case rtfCellPos:
		break;
	case rtfMergeRngFirst:
		break;
	case rtfMergePrevious:
		break;
	case rtfRowLeft:
		break;
	case rtfRowRight:
		break;
	case rtfRowCenter:
		break;
	case rtfRowLeftEdge:
		break;
	case rtfRowHt:
		break;
	case rtfRowHeader:
		break;
	case rtfRowKeep:
		break;
	case rtfRTLRow:
		break;
	case rtfLTRRow:
		break;
	case rtfRowBordTop:
		break;
	case rtfRowBordLeft:
		break;
	case rtfRowBordBottom:
		break;
	case rtfRowBordRight:
		break;
	case rtfRowBordHoriz:
		break;
	case rtfRowBordVert:
		break;
	case rtfCellBordBottom:
		break;
	case rtfCellBordTop:
		break;
	case rtfCellBordLeft:
		break;
	case rtfCellBordRight:
		break;
	case rtfCellShading:
		break;
	case rtfCellBgPatH:
		break;
	case rtfCellBgPatV:
		break;
	case rtfCellFwdDiagBgPat:
		break;
	case rtfCellBwdDiagBgPat:
		break;
	case rtfCellHatchBgPat:
		break;
	case rtfCellDiagHatchBgPat:
		break;
	case rtfCellDarkBgPatH:
		break;
	case rtfCellDarkBgPatV:
		break;
	case rtfCellFwdDarkBgPat:
		break;
	case rtfCellBwdDarkBgPat:
		break;
	case rtfCellDarkHatchBgPat:
		break;
	case rtfCellDarkDiagHatchBgPat:
		break;
	case rtfCellBgPatLineColor:
		break;
	case rtfCellBgPatColor:
		break;
	}
}


static void
ParAttr ()
{
	switch (rtfMinor)
	{
	case rtfParDef:
		break;
	case rtfStyleNum:
		break;
	case rtfHyphenate:
		break;
	case rtfInTable:
		break;
	case rtfKeep:
		break;
	case rtfNoWidowControl:
		break;
	case rtfKeepNext:
		break;
	case rtfOutlineLevel:
		break;
	case rtfNoLineNum:
		break;
	case rtfPBBefore:
		break;
	case rtfSideBySide:
		break;
	case rtfQuadLeft:
		break;
	case rtfQuadRight:
		break;
	case rtfQuadJust:
		break;
	case rtfQuadCenter:
		break;
	case rtfFirstIndent:
		break;
	case rtfLeftIndent:
		break;
	case rtfRightIndent:
		break;
	case rtfSpaceBefore:
		break;
	case rtfSpaceAfter:
		break;
	case rtfSpaceBetween:
		break;
	case rtfSpaceMultiply:
		break;
	case rtfSubDocument:
		break;
	case rtfRTLPar:
		break;
	case rtfLTRPar:
		break;
	case rtfTabPos:
		break;
	case rtfTabLeft:
		break;
	case rtfTabRight:
		break;
	case rtfTabCenter:
		break;
	case rtfTabDecimal:
		break;
	case rtfTabBar:
		break;
	case rtfLeaderDot:
		break;
	case rtfLeaderHyphen:
		break;
	case rtfLeaderUnder:
		break;
	case rtfLeaderThick:
		break;
	case rtfLeaderEqual:
		break;
	case rtfParLevel:
		break;
	case rtfParBullet:
		break;
	case rtfParSimple:
		break;
	case rtfParNumCont:
		break;
	case rtfParNumOnce:
		break;
	case rtfParNumAcross:
		break;
	case rtfParHangIndent:
		break;
	case rtfParNumRestart:
		break;
	case rtfParNumCardinal:
		break;
	case rtfParNumDecimal:
		break;
	case rtfParNumULetter:
		break;
	case rtfParNumURoman:
		break;
	case rtfParNumLLetter:
		break;
	case rtfParNumLRoman:
		break;
	case rtfParNumOrdinal:
		break;
	case rtfParNumOrdinalText:
		break;
	case rtfParNumBold:
		break;
	case rtfParNumItalic:
		break;
	case rtfParNumAllCaps:
		break;
	case rtfParNumSmallCaps:
		break;
	case rtfParNumUnder:
		break;
	case rtfParNumDotUnder:
		break;
	case rtfParNumDbUnder:
		break;
	case rtfParNumNoUnder:
		break;
	case rtfParNumWordUnder:
		break;
	case rtfParNumStrikethru:
		break;
	case rtfParNumForeColor:
		break;
	case rtfParNumFont:
		break;
	case rtfParNumFontSize:
		break;
	case rtfParNumIndent:
		break;
	case rtfParNumSpacing:
		break;
	case rtfParNumInclPrev:
		break;
	case rtfParNumCenter:
		break;
	case rtfParNumLeft:
		break;
	case rtfParNumRight:
		break;
	case rtfParNumStartAt:
		break;
	case rtfBorderTop:
		break;
	case rtfBorderBottom:
		break;
	case rtfBorderLeft:
		break;
	case rtfBorderRight:
		break;
	case rtfBorderBetween:
		break;
	case rtfBorderBar:
		break;
	case rtfBorderBox:
		break;
	case rtfBorderSingle:
		break;
	case rtfBorderThick:
		break;
	case rtfBorderShadow:
		break;
	case rtfBorderDouble:
		break;
	case rtfBorderDot:
		break;
	case rtfBorderDash:
		break;
	case rtfBorderHair:
		break;
	case rtfBorderWidth:
		break;
	case rtfBorderColor:
		break;
	case rtfBorderSpace:
		break;
	case rtfShading:
		break;
	case rtfBgPatH:
		break;
	case rtfBgPatV:
		break;
	case rtfFwdDiagBgPat:
		break;
	case rtfBwdDiagBgPat:
		break;
	case rtfHatchBgPat:
		break;
	case rtfDiagHatchBgPat:
		break;
	case rtfDarkBgPatH:
		break;
	case rtfDarkBgPatV:
		break;
	case rtfFwdDarkBgPat:
		break;
	case rtfBwdDarkBgPat:
		break;
	case rtfDarkHatchBgPat:
		break;
	case rtfDarkDiagHatchBgPat:
		break;
	case rtfBgPatLineColor:
		break;
	case rtfBgPatColor:
		break;
	}
}


/*
 * Several of the attributes can be turned off with param value
 * of zero (e.g., \b vs. \b0), but since the value of rtfParam
 * is 0 if no param is given, test the text of the token directly.
 */

static void
CharAttr ()
{
int	turnOn = (rtfTextBuf[rtfTextLen-1] != '0');

	switch (rtfMinor)
	{
	case rtfPlain:
		break;
	case rtfBold:
		break;
	case rtfAllCaps:
		break;
	case rtfDeleted:
		break;
	case rtfSubScript:
		break;
	case rtfSubScrShrink:
		break;
	case rtfNoSuperSub:
		break;
	case rtfExpand:
		break;
	case rtfExpandTwips:
		break;
	case rtfKerning:
		break;
	case rtfFontNum:
		break;
	case rtfFontSize:
		break;
	case rtfItalic:
		break;
	case rtfOutline:
		break;
	case rtfRevised:
		break;
	case rtfRevAuthor:
		break;
	case rtfRevDTTM:
		break;
	case rtfSmallCaps:
		break;
	case rtfShadow:
		break;
	case rtfStrikeThru:
		break;
	case rtfUnderline:
		break;
	case rtfDotUnderline:
		break;
	case rtfDbUnderline:
		break;
	case rtfNoUnderline:
		break;
	case rtfWordUnderline:
		break;
	case rtfSuperScript:
		break;
	case rtfSuperScrShrink:
		break;
	case rtfInvisible:
		break;
	case rtfForeColor:
		break;
	case rtfBackColor:
		break;
	case rtfRTLChar:
		break;
	case rtfLTRChar:
		break;
	case rtfCharStyleNum:
		break;
	case rtfCharCharSet:
		break;
	case rtfLanguage:
		SelectLanguage ();
		break;
	case rtfGray:
		break;
	}
}


static void
PictAttr ()
{
	switch (rtfMinor)
	{
	case rtfMacQD:
		break;
	case rtfPMMetafile:
		break;
	case rtfWinMetafile:
		break;
	case rtfDevIndBitmap:
		break;
	case rtfWinBitmap:
		break;
	case rtfPixelBits:
		break;
	case rtfBitmapPlanes:
		break;
	case rtfBitmapWid:
		break;
	case rtfPicWid:
		break;
	case rtfPicHt:
		break;
	case rtfPicGoalWid:
		break;
	case rtfPicGoalHt:
		break;
	case rtfPicScaleX:
		break;
	case rtfPicScaleY:
		break;
	case rtfPicScaled:
		break;
	case rtfPicCropTop:
		break;
	case rtfPicCropBottom:
		break;
	case rtfPicCropLeft:
		break;
	case rtfPicCropRight:
		break;
	case rtfPicMFHasBitmap:
		break;
	case rtfPicMFBitsPerPixel:
		break;
	case rtfPicBinary:
		break;
	}
}


static void
BookmarkAttr ()
{
	switch (rtfMinor)
	{
	case rtfBookmarkFirstCol:
		break;
	case rtfBookmarkLastCol:
		break;
	}
}


static void
NeXTGrAttr ()
{
	switch (rtfMinor)
	{
	case rtfNeXTGWidth:
		break;
	case rtfNeXTGHeight:
		break;
	}
}


static void
FieldAttr ()
{
	switch (rtfMinor)
	{
	case rtfFieldDirty:
		break;
	case rtfFieldEdited:
		break;
	case rtfFieldLocked:
		break;
	case rtfFieldPrivate:
		break;
	case rtfFieldAlt:
		break;
	}
}


static void
TOCAttr ()
{
	switch (rtfMinor)
	{
	case rtfTOCType:
		break;
	case rtfTOCLevel:
		break;
	}
}


static void
PosAttr ()
{
	switch (rtfMinor)
	{
	case rtfAbsWid:
		break;
	case rtfAbsHt:
		break;
	case rtfRPosMargH:
		break;
	case rtfRPosPageH:
		break;
	case rtfRPosColH:
		break;
	case rtfPosX:
		break;
	case rtfPosNegX:
		break;
	case rtfPosXCenter:
		break;
	case rtfPosXInside:
		break;
	case rtfPosXOutSide:
		break;
	case rtfPosXRight:
		break;
	case rtfPosXLeft:
		break;
	case rtfRPosMargV:
		break;
	case rtfRPosPageV:
		break;
	case rtfRPosParaV:
		break;
	case rtfPosY:
		break;
	case rtfPosNegY:
		break;
	case rtfPosYInline:
		break;
	case rtfPosYTop:
		break;
	case rtfPosYCenter:
		break;
	case rtfPosYBottom:
		break;
	case rtfNoWrap:
		break;
	case rtfDistFromTextAll:
		break;
	case rtfDistFromTextX:
		break;
	case rtfDistFromTextY:
		break;
	case rtfTextDistY:
		break;
	case rtfDropCapLines:
		break;
	case rtfDropCapType:
		break;
	}
}


static void
ObjAttr ()
{
	switch (rtfMinor)
	{
	case rtfObjEmb:
		break;
	case rtfObjLink:
		break;
	case rtfObjAutoLink:
		break;
	case rtfObjSubscriber:
		break;
	case rtfObjPublisher:
		break;
	case rtfObjICEmb:
		break;
	case rtfObjLinkSelf:
		break;
	case rtfObjLock:
		break;
	case rtfObjUpdate:
		break;
	case rtfObjHt:
		break;
	case rtfObjWid:
		break;
	case rtfObjSetSize:
		break;
	case rtfObjAlign:
		break;
	case rtfObjTransposeY:
		break;
	case rtfObjCropTop:
		break;
	case rtfObjCropBottom:
		break;
	case rtfObjCropLeft:
		break;
	case rtfObjCropRight:
		break;
	case rtfObjScaleX:
		break;
	case rtfObjScaleY:
		break;
	case rtfObjResRTF:
		break;
	case rtfObjResPict:
		break;
	case rtfObjResBitmap:
		break;
	case rtfObjResText:
		break;
	case rtfObjResMerge:
		break;
	case rtfObjBookmarkPubObj:
		break;
	case rtfObjPubAutoUpdate:
		break;
	}
}


static void
FNoteAttr ()
{
	switch (rtfMinor)
	{
	case rtfFNAlt:
		break;
	}
}


static void
KeyCodeAttr ()
{
	switch (rtfMinor)
	{
	case rtfAltKey:
		break;
	case rtfShiftKey:
		break;
	case rtfControlKey:
		break;
	case rtfFunctionKey:
		break;
	}
}


static void
ACharAttr ()
{
	switch (rtfMinor)
	{
	case rtfACBold:
		break;
	case rtfACAllCaps:
		break;
	case rtfACForeColor:
		break;
	case rtfACSubScript:
		break;
	case rtfACExpand:
		break;
	case rtfACFontNum:
		break;
	case rtfACFontSize:
		break;
	case rtfACItalic:
		break;
	case rtfACLanguage:
		break;
	case rtfACOutline:
		break;
	case rtfACSmallCaps:
		break;
	case rtfACShadow:
		break;
	case rtfACStrikeThru:
		break;
	case rtfACUnderline:
		break;
	case rtfACDotUnderline:
		break;
	case rtfACDbUnderline:
		break;
	case rtfACNoUnderline:
		break;
	case rtfACWordUnderline:
		break;
	case rtfACSuperScript:
		break;
	}
}


static void
FontAttr ()
{
	switch (rtfMinor)
	{
	case rtfFontCharSet:
		break;
	case rtfFontPitch:
		break;
	case rtfFontCodePage:
		break;
	case rtfFTypeNil:
		break;
	case rtfFTypeTrueType:
		break;
	}
}


static void
FileAttr ()
{
	switch (rtfMinor)
	{
	case rtfFileNum:
		break;
	case rtfFileRelPath:
		break;
	case rtfFileOSNum:
		break;
	}
}


static void
FileSource ()
{
	switch (rtfMinor)
	{
	case rtfSrcMacintosh:
		break;
	case rtfSrcDOS:
		break;
	case rtfSrcNTFS:
		break;
	case rtfSrcHPFS:
		break;
	case rtfSrcNetwork:
		break;
	}
}


static void
DrawAttr ()
{
	switch (rtfMinor)
	{
	case rtfDrawLock:
		break;
	case rtfDrawPageRelX:
		break;
	case rtfDrawColumnRelX:
		break;
	case rtfDrawMarginRelX:
		break;
	case rtfDrawPageRelY:
		break;
	case rtfDrawColumnRelY:
		break;
	case rtfDrawMarginRelY:
		break;
	case rtfDrawHeight:
		break;
	case rtfDrawBeginGroup:
		break;
	case rtfDrawGroupCount:
		break;
	case rtfDrawEndGroup:
		break;
	case rtfDrawArc:
		break;
	case rtfDrawCallout:
		break;
	case rtfDrawEllipse:
		break;
	case rtfDrawLine:
		break;
	case rtfDrawPolygon:
		break;
	case rtfDrawPolyLine:
		break;
	case rtfDrawRect:
		break;
	case rtfDrawTextBox:
		break;
	case rtfDrawOffsetX:
		break;
	case rtfDrawSizeX:
		break;
	case rtfDrawOffsetY:
		break;
	case rtfDrawSizeY:
		break;
	case rtfCOAngle:
		break;
	case rtfCOAccentBar:
		break;
	case rtfCOBestFit:
		break;
	case rtfCOBorder:
		break;
	case rtfCOAttachAbsDist:
		break;
	case rtfCOAttachBottom:
		break;
	case rtfCOAttachCenter:
		break;
	case rtfCOAttachTop:
		break;
	case rtfCOLength:
		break;
	case rtfCONegXQuadrant:
		break;
	case rtfCONegYQuadrant:
		break;
	case rtfCOOffset:
		break;
	case rtfCOAttachSmart:
		break;
	case rtfCODoubleLine:
		break;
	case rtfCORightAngle:
		break;
	case rtfCOSingleLine:
		break;
	case rtfCOTripleLine:
		break;
	case rtfDrawTextBoxMargin:
		break;
	case rtfDrawTextBoxText:
		break;
	case rtfDrawRoundRect:
		break;
	case rtfDrawPointX:
		break;
	case rtfDrawPointY:
		break;
	case rtfDrawPolyCount:
		break;
	case rtfDrawArcFlipX:
		break;
	case rtfDrawArcFlipY:
		break;
	case rtfDrawLineBlue:
		break;
	case rtfDrawLineGreen:
		break;
	case rtfDrawLineRed:
		break;
	case rtfDrawLinePalette:
		break;
	case rtfDrawLineDashDot:
		break;
	case rtfDrawLineDashDotDot:
		break;
	case rtfDrawLineDash:
		break;
	case rtfDrawLineDot:
		break;
	case rtfDrawLineGray:
		break;
	case rtfDrawLineHollow:
		break;
	case rtfDrawLineSolid:
		break;
	case rtfDrawLineWidth:
		break;
	case rtfDrawHollowEndArrow:
		break;
	case rtfDrawEndArrowLength:
		break;
	case rtfDrawSolidEndArrow:
		break;
	case rtfDrawEndArrowWidth:
		break;
	case rtfDrawHollowStartArrow:
		break;
	case rtfDrawStartArrowLength:
		break;
	case rtfDrawSolidStartArrow:
		break;
	case rtfDrawStartArrowWidth:
		break;
	case rtfDrawBgFillBlue:
		break;
	case rtfDrawBgFillGreen:
		break;
	case rtfDrawBgFillRed:
		break;
	case rtfDrawBgFillPalette:
		break;
	case rtfDrawBgFillGray:
		break;
	case rtfDrawFgFillBlue:
		break;
	case rtfDrawFgFillGreen:
		break;
	case rtfDrawFgFillRed:
		break;
	case rtfDrawFgFillPalette:
		break;
	case rtfDrawFgFillGray:
		break;
	case rtfDrawFillPatIndex:
		break;
	case rtfDrawShadow:
		break;
	case rtfDrawShadowXOffset:
		break;
	case rtfDrawShadowYOffset:
		break;
	}
}


static void
IndexAttr ()
{
	switch (rtfMinor)
	{
	case rtfIndexNumber:
		break;
	case rtfIndexBold:
		break;
	case rtfIndexItalic:
		break;
	}
}


static void
SelectLanguage ()
{
	switch (rtfParam)
	{
	case rtfLangNoLang:
		break;
	case rtfLangAlbanian:
		break;
	case rtfLangArabic:
		break;
	case rtfLangBahasa:
		break;
	case rtfLangBelgianDutch:
		break;
	case rtfLangBelgianFrench:
		break;
	case rtfLangBrazilianPortuguese:
		break;
	case rtfLangBulgarian:
		break;
	case rtfLangCatalan:
		break;
	case rtfLangLatinCroatoSerbian:
		break;
	case rtfLangCzech:
		break;
	case rtfLangDanish:
		break;
	case rtfLangDutch:
		break;
	case rtfLangAustralianEnglish:
		break;
	case rtfLangUKEnglish:
		break;
	case rtfLangUSEnglish:
		break;
	case rtfLangFinnish:
		break;
	case rtfLangFrench:
		break;
	case rtfLangCanadianFrench:
		break;
	case rtfLangGerman:
		break;
	case rtfLangGreek:
		break;
	case rtfLangHebrew:
		break;
	case rtfLangHungarian:
		break;
	case rtfLangIcelandic:
		break;
	case rtfLangItalian:
		break;
	case rtfLangJapanese:
		break;
	case rtfLangKorean:
		break;
	case rtfLangBokmalNorwegian:
		break;
	case rtfLangNynorskNorwegian:
		break;
	case rtfLangPolish:
		break;
	case rtfLangPortuguese:
		break;
	case rtfLangRhaetoRomanic:
		break;
	case rtfLangRomanian:
		break;
	case rtfLangRussian:
		break;
	case rtfLangCyrillicSerboCroatian:
		break;
	case rtfLangSimplifiedChinese:
		break;
	case rtfLangSlovak:
		break;
	case rtfLangCastilianSpanish:
		break;
	case rtfLangMexicanSpanish:
		break;
	case rtfLangSwedish:
		break;
	case rtfLangSwissFrench:
		break;
	case rtfLangSwissGerman:
		break;
	case rtfLangSwissItalian:
		break;
	case rtfLangThai:
		break;
	case rtfLangTraditionalChinese:
		break;
	case rtfLangTurkish:
		break;
	case rtfLangUrdu:
		break;
	}
}
